@@ -145,9 +145,7 @@ This is just a zip file beginning with the magic string "C<PK\003\004>".
 
 The pre-computed cache name.  A pack('Z40') string of the value of -T 
 (--tempcache) or the hash of the file, followed by C<\0CACHE>.  The hash
-of the file is calculated with L<Digest::SHA>, L<Digest::SHA1>, or 
-L<Digest::MD5>.  If none of those modules is available, the C<mtime> of
-the file is used.
+of the file is calculated with L<Digest::SHA>.
 
 A pack('N') number of the total length of FILE and PAR sections,
 followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
@@ -210,16 +208,6 @@ BEGIN {
 
 _par_init_env();
 
-if (exists $ENV{PAR_ARGV_0} and $ENV{PAR_ARGV_0} ) {
-    @ARGV = map $ENV{"PAR_ARGV_$_"}, (1 .. $ENV{PAR_ARGC} - 1);
-    $0 = $ENV{PAR_ARGV_0};
-}
-else {
-    for (keys %ENV) {
-        delete $ENV{$_} if /^PAR_ARGV_/;
-    }
-}
-
 my $quiet = !$ENV{PAR_DEBUG};
 
 # fix $progname if invoked from PATH
@@ -253,7 +241,7 @@ my ($start_pos, $data_pos);
         my $nread = read _FH, $buf, $offset;
         die qq[read failed on "$progname": $!] unless $nread == $offset;
         $idx = rindex($buf, "\nPAR.pm\n");
-        last if $idx >= 0 || $offset == $size || $offset > 128 * 1024;
+        last if $idx >= 0 || $offset == $size || $offset > 256 * 1024;
         $offset *= 2;
     }
     last unless $idx >= 0;
@@ -284,24 +272,13 @@ my ($start_pos, $data_pos);
         read _FH, $buf, unpack("N", $buf);
 
         if (defined($ext) and $ext !~ /\.(?:pm|pl|ix|al)$/i) {
-            my ($out, $filename) = _tempfile($ext, $crc);
-            if ($out) {
-                binmode($out);
-                print $out $buf;
-                close $out;
-                chmod 0755, $filename;
-            }
+            my $filename = _tempfile("$crc$ext", $buf, 0755);
             $PAR::Heavy::FullCache{$fullname} = $filename;
             $PAR::Heavy::FullCache{$filename} = $fullname;
         }
         elsif ( $fullname =~ m|^/?shlib/| and defined $ENV{PAR_TEMP} ) {
-            # should be moved to _tempfile()
-            my $filename = "$ENV{PAR_TEMP}/$basename$ext";
+            my $filename = _tempfile("$basename$ext", $buf, 0755);
             outs("SHLIB: $filename\n");
-            open my $out, '>', $filename or die $!;
-            binmode($out);
-            print $out $buf;
-            close $out;
         }
         else {
             $require_list{$fullname} =
@@ -320,32 +297,21 @@ my ($start_pos, $data_pos);
 
         return if ref $module or !$module;
 
-        my $filename = delete $require_list{$module} || do {
-            my $key;
-            foreach (keys %require_list) {
-                next unless /\Q$module\E$/;
-                $key = $_; last;
-            }
-            delete $require_list{$key} if defined($key);
-        } or return;
+        my $info = delete $require_list{$module} or return;
 
-        $INC{$module} = "/loader/$filename/$module";
+        $INC{$module} = "/loader/$info/$module";
 
         if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
             my $fh = IO::File->new_tmpfile or die $!;
             binmode($fh);
-            print $fh $filename->{buf};
+            print $fh $info->{buf};
             seek($fh, 0, 0);
             return $fh;
         }
         else {
-            my ($out, $name) = _tempfile('.pm', $filename->{crc});
-            if ($out) {
-                binmode($out);
-                print $out $filename->{buf};
-                close $out;
-            }
-            open my $fh, '<', $name or die $!;
+            my $filename = _tempfile("$info->{crc}.pm", $info->{buf});
+
+            open my $fh, '<', $filename or die "can't read $filename: $!";
             binmode($fh);
             return $fh;
         }
@@ -472,6 +438,7 @@ if ($out) {
         #local $INC{'Cwd.pm'} = __FILE__ if $^O ne 'MSWin32';
         require IO::File;
         require Archive::Zip;
+        require Digest::SHA;
     }
 
     my $par = shift(@ARGV);
@@ -543,28 +510,23 @@ if ($out) {
     if ($bundle) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
+
         init_inc();
 
         require_modules();
 
-        my @inc = sort {
-            length($b) <=> length($a)
-        } grep {
-            !/BSDPAN/
-        } grep {
-            ($bundle ne 'site') or
-            ($_ ne $Config::Config{archlibexp} and
-             $_ ne $Config::Config{privlibexp});
-        } @INC;
+        my @inc = grep { !/BSDPAN/ } 
+                       grep {
+                           ($bundle ne 'site') or
+                           ($_ ne $Config::Config{archlibexp} and
+                           $_ ne $Config::Config{privlibexp});
+                       } @INC;
 
-        # File exists test added to fix RT #41790:
-        # Funny, non-existing entry in _<....auto/Compress/Raw/Zlib/autosplit.ix.
-        # This is a band-aid fix with no deeper grasp of the issue.
-        # Somebody please go through the pain of understanding what's happening,
-        # I failed. -- Steffen
+        # Now determine the files loaded above by require_modules():
+        # Perl source files are found in values %INC and DLLs are
+        # found in @DynaLoader::dl_shared_objects.
         my %files;
-        /^_<(.+)$/ and -e $1 and $files{$1}++ for keys %::;
-        $files{$_}++ for values %INC;
+        $files{$_}++ for @DynaLoader::dl_shared_objects, values %INC;
 
         my $lib_ext = $Config::Config{lib_ext};
         my %written;
@@ -633,24 +595,13 @@ if ($out) {
 
     $cache_name = substr $cache_name, 0, 40;
     if (!$cache_name and my $mtime = (stat($out))[9]) {
-        my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
-            || eval { require Digest::SHA1; Digest::SHA1->new }
-            || eval { require Digest::MD5; Digest::MD5->new };
+        my $ctx = Digest::SHA->new(1);
+        open(my $fh, "<", $out);
+        binmode($fh);
+        $ctx->addfile($fh);
+        close($fh);
 
-        # Workaround for bug in Digest::SHA 5.38 and 5.39
-        my $sha_version = eval { $Digest::SHA::VERSION } || 0;
-        if ($sha_version eq '5.38' or $sha_version eq '5.39') {
-            $ctx->addfile($out, "b") if ($ctx);
-        }
-        else {
-            if ($ctx and open(my $fh, "<$out")) {
-                binmode($fh);
-                $ctx->addfile($fh);
-                close($fh);
-            }
-        }
-
-        $cache_name = $ctx ? $ctx->hexdigest : $mtime;
+        $cache_name = $ctx->hexdigest;
     }
     $cache_name .= "\0" x (41 - length $cache_name);
     $cache_name .= "CACHE";
@@ -681,6 +632,13 @@ if ($out) {
         require File::Find;
         require Archive::Zip;
     }
+
+    # increase the chunk size for Archive::Zip so that it will find the EOCD
+    # even if more stuff (OS-specific codesigning, for example) has been appended to the pp exe
+    # OSX codesign tool appends at least 180K to a binary and so make the ChunkSize generously
+    # greater than this
+    Archive::Zip::setChunkSize(256*1024);
+
     my $zip = Archive::Zip->new;
     my $fh = IO::File->new;
     $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
@@ -763,6 +721,7 @@ sub require_modules {
     require IO::File;
     require Compress::Zlib;
     require Archive::Zip;
+    require Digest::SHA;
     require PAR;
     require PAR::Heavy;
     require PAR::Dist;
@@ -775,6 +734,7 @@ sub require_modules {
     eval { require Archive::Unzip::Burst };
     eval { require Tie::Hash::NamedCapture };
     eval { require PerlIO; require PerlIO::scalar };
+    eval { require utf8 };
 }
 
 # The C version of this code appears in myldr/mktmpdir.c
@@ -819,24 +779,14 @@ sub _set_par_temp {
                 $stmpdir .= "$Config{_delim}cache-" . $buf;
             }
             else {
-                my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
-                    || eval { require Digest::SHA1; Digest::SHA1->new }
-                    || eval { require Digest::MD5; Digest::MD5->new };
+                require Digest::SHA; 
+                my $ctx = Digest::SHA->new(1);
+                open(my $fh, "<", $progname);
+                binmode($fh);
+                $ctx->addfile($fh);
+                close($fh);
 
-                # Workaround for bug in Digest::SHA 5.38 and 5.39
-                my $sha_version = eval { $Digest::SHA::VERSION } || 0;
-                if ($sha_version eq '5.38' or $sha_version eq '5.39') {
-                    $ctx->addfile($progname, "b") if ($ctx);
-                }
-                else {
-                    if ($ctx and open(my $fh, "<$progname")) {
-                        binmode($fh);
-                        $ctx->addfile($fh);
-                        close($fh);
-                    }
-                }
-
-                $stmpdir .= "$Config{_delim}cache-" . ( $ctx ? $ctx->hexdigest : $mtime );
+                $stmpdir .= "$Config{_delim}cache-" . $ctx->hexdigest;
             }
             close($fh);
         }
@@ -853,23 +803,33 @@ sub _set_par_temp {
     $par_temp = $1 if $ENV{PAR_TEMP} and $ENV{PAR_TEMP} =~ /(.+)/;
 }
 
+
+# check if $name (relative to $par_temp) already exists;
+# if not, create a file with a unique temporary name, 
+# fill it with $contents, set its file mode to $mode if present;
+# finaly rename it to $name; 
+# in any case return the absolute filename
 sub _tempfile {
-    my ($ext, $crc) = @_;
-    my ($fh, $filename);
+    my ($name, $contents, $mode) = @_;
 
-    $filename = "$par_temp/$crc$ext";
+    my $fullname = "$par_temp/$name";
+    unless (-e $fullname) {
+        my $tempname = "$fullname.$$";
 
-    if ($ENV{PAR_CLEAN}) {
-        unlink $filename if -e $filename;
-        push @tmpfile, $filename;
-    }
-    else {
-        return (undef, $filename) if (-r $filename);
+        open my $fh, '>', $tempname or die "can't write $tempname: $!";
+        binmode $fh;
+        print $fh $contents;
+        close $fh;
+        chmod $mode, $tempname if defined $mode;
+
+        rename($tempname, $fullname) or unlink($tempname);
+        # NOTE: The rename() error presumably is something like ETXTBSY 
+        # (scenario: another process was faster at extraction $fullname
+        # than us and is already using it in some way); anyway, 
+        # let's assume $fullname is "good" and clean up our copy.
     }
 
-    open $fh, '>', $filename or die $!;
-    binmode($fh);
-    return($fh, $filename);
+    return $fullname;
 }
 
 # same code lives in PAR::SetupProgname::set_progname
@@ -926,7 +886,7 @@ sub _par_init_env {
         $ENV{PAR_INITIALIZED} = 2;
     }
 
-    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ARGC ARGV_0 ) ) {
+    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ) ) {
         delete $ENV{'PAR_'.$_};
     }
     for (qw/ TMPDIR TEMP CLEAN DEBUG /) {
@@ -997,11 +957,11 @@ L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-L<http://par.perl.org/> is the official PAR website.  You can write
+You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
-Please submit bug reports to E<lt>bug-par@rt.cpan.orgE<gt>.
+Please submit bug reports to E<lt>bug-par-packer@rt.cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
@@ -1024,7 +984,7 @@ under the Artistic License.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+See F<LICENSE>.
 
 =cut
 
